# multiClust

multiClust: An R-package for Identifying Biologically Relevant Clusters in Cancer Transcriptome Profiles

__Installation:__

To install this package, start R and enter:

`if (!requireNamespace("BiocManager", quietly=TRUE))`
    `install.packages("BiocManager")`

`BiocManager::install("multiClust")`

__Tutorial__:

For instructions on how to use multiClust, please see the tutorial available at: https://bioconductor.org/packages/release/bioc/vignettes/multiClust/inst/doc/multiClust.html

__More Information__:

For more information about multiClust, please see the associated publication at:
https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4907340/
